package util.hlist.immutable;

/**
 * Nachbildung der Scala-HList (heterogene Liste, Liste mit unterschiedlichem Datentyp je Position)
 * mit Java-Generics.
 * 
 * @author Heiner K&uuml;cker
 */
public final class JavaHList
{
	/**
	 * Konstruktor private,
	 * Instanziieren verboten.
	 */
	private JavaHList()
	{
		// absichtlich leer
	}
	
	/**
	 * Get Element von Index 0.
	 * 
	 * @param head Liste-Kopf-Element
	 * @return Wert von Listen-Element Index 0 (vom ersten Entry)
	 */
	public static <T> T get0(
			final AbstractHListEntry<T, ?> head )
	{
		return head.value;
	}
	
	/**
	 * Get Element von Index 1.
	 * 
	 * @param head Liste-Kopf-Element
	 * @return Wert von Listen-Element Index 1 (vom zweiten Entry)
	 */
	public static <T> T get1(
			final AbstractHListEntry<?, ? extends AbstractHListEntry<T, ?>> head )
	{
		return head.nextEntry.value;
	}
	
	/**
	 * Get Element von Index 2.
	 * 
	 * @param head Liste-Kopf-Element
	 * @return Wert von Listen-Element Index 2 (vom dritten Entry)
	 */
	public static <T> T get2(
			final AbstractHListEntry<?, ? extends HListEntry<?, ? extends AbstractHListEntry<T, ?>>> head )
	{
		return head.nextEntry.nextEntry.value;
	}
	
	/**
	 * Get Element von Index 3.
	 * 
	 * @param head Liste-Kopf-Element
	 * @return Wert von Listen-Element Index 3 (vom vierten Entry)
	 */
	public static <T> T get3(
			final AbstractHListEntry<?, ? extends HListEntry<?, ? extends HListEntry<?, ? extends AbstractHListEntry<T, ?>>>> head )
	{
		return head.nextEntry.nextEntry.nextEntry.value;
	}
	
	/**
	 * Get Element von Index 4.
	 * 
	 * @param head Liste-Kopf-Element
	 * @return Wert von Listen-Element Index 4 (vom fnften Entry)
	 */
	public static <T> T get4(
			final AbstractHListEntry<?, ? extends HListEntry<?, ? extends HListEntry<?, ? extends HListEntry<?, ? extends AbstractHListEntry<T, ?>>>>> head )
	{
		return head.nextEntry.nextEntry.nextEntry.nextEntry.value;
	}
	
}
