package util.hlist.mutable;

/**
 * Nachbildung Scala HList
 * (heterogene Liste,
 *  Liste mit unterschiedlichen
 *  Typen je Position, aber trotzdem
 *  typsicher)
 * mit Java-Generics.
 * <br/><br/>
 * Bei dieser Variante ist der
 * Wert vernderlich, die Struktur
 * der heterogenen Liste ist
 * aber unvernderlich.
 * 
 * @param <T> Type-Parameter, Type des aktuellen Wertes
 * @param <N> Type-Parameter, Type des Nachfolger-Entry
 * @author Heiner K&uuml;cker
 */
public abstract class AbstractMutableHListEntry<T, N extends AbstractMutableHListEntry<?, ?>>
{
	/**
	 * aktueller Wert
	 */
	public T value;

	/**
	 * Nchstes Entry-Objekt,
	 * realisiert eine einfach
	 * verlinkte Liste.
	 */
	public final N nextEntry;

	/**
	 * @param nextEntry
	 */
	protected AbstractMutableHListEntry(
			final N nextEntry )
	{
		this.nextEntry = nextEntry;
	}

	/**
	 * @param value
	 * @param nextEntry
	 */
	protected AbstractMutableHListEntry(
			final T value ,
			final N nextEntry )
	{
		this.value = value;
		this.nextEntry = nextEntry;
	}
	
}
