package util.iterator.recursiv;

import java.util.Iterator;

/**
 * Klasse fr Rekursions-Terminierung
 * der Iterator-Schachtelung aus
 * {@link RecursivOrTerminalIterator}en.
 * 
 * @param <E> Type-Parameter, Element-Type
 * @author Heiner K&uuml;cker
 */
public final class TerminalIterator<E>
implements RecursivOrTerminalIterator<E>
{
    /**
     * untergeordneter Iterator.
     */
	private final Iterator<E> innerIterator;

	/**
	 * Konstruktor.
	 * 
	 * @param innerIterator untergeordneter Iterator
	 */
	public TerminalIterator(
			final Iterator<E> innerIterator )
	{
		this.innerIterator = innerIterator;
	}

	/**
	 * @see java.util.Iterator#hasNext()
	 */
	@Override
	public boolean hasNext()
	{
		return this.innerIterator.hasNext();
	}

	/**
	 * @see java.util.Iterator#next()
	 */
	@Override
	public E next()
	{
		return this.innerIterator.next();
	}

	/**
	 * @see java.util.Iterator#remove()
	 */
	@Override
	public void remove()
	{
		this.innerIterator.remove();
	}

}
