package util.yield_for.nested;

import java.util.NoSuchElementException;

/**
 * Implementierungsklasse fr eine
 * einstufige For-Schleife mit 
 * yield-return.
 * <br/>
 * Die Hierarchie der For-Schleifen
 * endet mit dieser Klasse.
 * 
 * @param <I> Type-Parameter, Typ der Laufvariable der ersten Stufe, traditionsgemss I genannt
 * @param <R> Type-Parameter, Typ der Return-Wertes des yield-return
 * @author Heiner K&uuml;cker
 */
abstract public class FinalYieldFor1<I, R>
extends NestedOrFinalYieldFor1<I, R>
{
	/**
	 * Interner Merker ob die Initialsierung
	 * in der Methode {@link #hasNext()}
	 * erfolgt ist.
	 */
	private boolean isInitialized = false;
	
	/**
	 * Konstruktor.
	 */
	protected FinalYieldFor1()
	{
		super();
	}

	/**
	 * @see java.util.Iterator#hasNext()
	 */
	@Override
	public final boolean hasNext()
	{
		if ( ! this.isInitialized )
		{
			this.currentLoopValueLevel1 =
					initialValue();

			this.isInitialized = true;
		}

		return is(
				this.currentLoopValueLevel1 );
	}

	/**
	 * @see java.util.Iterator#next()
	 */
	@Override
	public final R next()
	{
		if ( ! hasNext() )
		{
			throw new NoSuchElementException();
		}
		final R yieldReturnValue =
				this.yieldReturn(
						this.currentLoopValueLevel1 );

		this.currentLoopValueLevel1 =
				skip(
						this.currentLoopValueLevel1 );
		
		return yieldReturnValue;
	}

	/**
	 * @see java.util.Iterator#remove()
	 * @throws UnsupportedOperationException always
	 */
	@Override
	public final void remove()
	{
		throw new UnsupportedOperationException();
	}

}
