package util.yield_for.nested;

import java.util.Iterator;

/**
 * Abstrakte Oberklasse fr
 * die erste Stufe einer
 * For-Schleife mit yield-return.
 * <br/>
 * Wie bei einem algebraischen Datentyp
 * kann die Hierarchie der For-Schleifen
 * enden ({@link FinalYieldFor1} oder
 * fortgesetzt werden 
 * {@link NestedYieldFor1}.
 * 
 * @param <I> Type-Parameter, Typ der Laufvariable der ersten Stufe, traditionsgemss I genannt
 * @param <R> Type-Parameter, Typ der Return-Wertes des yield-return
 * @author Heiner K&uuml;cker
 */
abstract public class NestedOrFinalYieldFor1<I, R>
//das Interface Iterator wird nur auf der obersten Ebene (1) implementiert
implements Iterator<R>
{
	/**
	 * aktuelle Laufvariable
	 */
	/*default*/ I currentLoopValueLevel1;
	
	/**
	 * Konstruktor.
	 */
	protected NestedOrFinalYieldFor1()
	{
		super();
		
		this.currentLoopValueLevel1 =
				initialValue();
	}

	/**
	 * Zu implementierende Methode, 
	 * welche den initialen Wert der 
	 * Laufvariable Zurckgibt.
	 * <br/>
	 * Dies entspricht dem linken
	 * Teil in den runden for-Klammern.
	 * 
	 * @return initialer Wert der Laufvariable
	 */
	abstract protected I initialValue();
	
	/**
	 * Zu implementierende Methode
	 * mit dem Code zum Verndern
	 * der Bedingung fr den nchsten
	 * Durchlauf oder Abbruch der
	 * For-Schleife.
	 * <br/>
	 * Dies entspricht dem mittleren
	 * Teil in den runden for-Klammern.
	 * 
	 * @param currentLoopValueLevel1 aktuelles Element, Laufvariable vor der Operation
	 * @return ob die For-Schleife fortgesetzt werden soll
	 */
	abstract protected boolean is(
			final I currentLoopValueLevel1 );
	
	/**
	 * Zu implementierende Methode
	 * mit dem Code der Operation
	 * zum Verndern der Laufvariable
	 * und fr den nchsten
	 * Durchlauf.
	 * <br/>
	 * Dies entspricht dem rechten
	 * Teil in den runden for-Klammern.
	 * 
	 * @param currentLoopValueLevel1 aktuelles Element, Laufvariable vor der Operation
	 * @return aktuelles Element, Laufvariable nach der Operation
	 */
	abstract protected I skip(
			final I currentLoopValueLevel1 );
	
	/**
	 * Zu implementierende Methode
	 * mit dem Code zum Zurckgeben
	 * des Ergebniswertes der aktuellen
	 * For-Schleife.
	 * <br/>
	 * Dies entspricht dem Teil 
	 * in den geschwungenen for-Klammern.
	 * 
	 * @param currentLoopValueLevel1 Lauf-Variable der ersten Ebene der For-Schleifen-Hierarchie
	 * @return Ergebnis-Wert des aktuellen Durchlaufes der For-Schleife
	 */
	abstract protected R yieldReturn(
			final I currentLoopValueLevel1 );
	
}
