package util.yield_for.nested;

import java.util.Iterator;

/**
 * Abstrakte Oberklasse fr
 * die zweite Stufe einer
 * For-Schleife mit yield-return.
 * <br/>
 * Wie bei einem algebraischen Datentyp
 * kann die Hierarchie der For-Schleifen
 * enden ({@link FinalYieldFor2} oder
 * fortgesetzt werden 
 * {@link NestedYieldFor2}.
 * 
 * @param <I> Type-Parameter, Typ der Laufvariable der ersten Stufe, traditionsgemss I genannt
 * @param <J> Type-Parameter, Typ der Laufvariable der zweiten Stufe, traditionsgemss J genannt
 * @param <R> Type-Parameter, Typ der Return-Wertes des yield-return
 * @author Heiner K&uuml;cker
 */
abstract public class NestedOrFinalYieldFor2<I, J, R>
{
	/**
	 * aktuelle Laufvariable
	 */
	/*default*/ J currentLoopValueLevel2;

	/**
	 * Konstruktor.
	 */
	protected NestedOrFinalYieldFor2()
	{
		super();
		
		init();
	}

	/**
	 * Methode zum Vorbereiten 
	 * eines neuen Durchlaufs,
	 * dies bricht mit dem
	 * Verhalten des normalen
	 * {@link java.util.Iterator}s.
	 */
	protected final void init()
	{
		this.currentLoopValueLevel2 =
				initialValue();
	}

	/**
	 * Zu implementierende Methode, 
	 * welche den initialen Wert der 
	 * Laufvariable Zurckgibt.
	 * <br/>
	 * Dies entspricht dem linken
	 * Teil in den runden for-Klammern.
	 * 
	 * @return initialer Wert der Laufvariable
	 */
	abstract protected J initialValue();

	/**
	 * Methode vorgesehen zum Aufruf aus
	 * {@link NestedYieldFor1#yieldReturn}.
	 * 
	 * @param currentLoopValueLevel1 Lauf-Variable der ersten Ebene der For-Schleifen-Hierarchie
	 * @return Ergebnis-Wert des aktuellen Durchlaufes der For-Schleife
	 */
	protected final R outerYieldReturn(
			final I currentLoopValueLevel1 )
	{
		return yieldReturn(
				currentLoopValueLevel1 ,
				this.currentLoopValueLevel2 );
	}

	/**
	 * Methode vorgesehen zum Aufruf aus
	 * {@link NestedYieldFor1}.
	 * <br/>
	 * Diese Methode ist in dieser
	 * Ebene der Ersatz fr
	 * {@link Iterator#hasNext}.
	 * 
	 * @return ob noch weitere Daten da sind
	 */
	abstract protected boolean outerIs();

	/**
	 * Zu implementierende Methode
	 * mit dem Code zum Verndern
	 * der Bedingung fr den nchsten
	 * Durchlauf oder Abbruch der
	 * For-Schleife.
	 * <br/>
	 * Dies entspricht dem mittleren
	 * Teil in den runden for-Klammern.
	 * 
	 * @param currentLoopValueLevel2 aktuelles Element, Laufvariable vor der Operation
	 * @return ob die For-Schleife fortgesetzt werden soll
	 */
	abstract protected boolean is(
			final J currentLoopValueLevel2 );

	/**
	 * Methode vorgesehen zum Aufruf aus
	 * {@link NestedYieldFor1}.
	 */
	abstract protected void outerSkip();

	/**
	 * Zu implementierende Methode
	 * mit dem Code der Operation
	 * zum Verndern der Laufvariable
	 * und fr den nchsten
	 * Durchlauf.
	 * <br/>
	 * Dies entspricht dem mittleren
	 * Teil in den runden for-Klammern.
	 * 
	 * @param currentLoopValueLevel2 aktuelles Element, Laufvariable vor der Operation
	 * @return aktuelles Element, Laufvariable nach der Operation
	 */
	abstract protected J skip(
			final J currentLoopValueLevel2 );
	
	/**
	 * Zu implementierende Methode
	 * mit dem Code zum Zurckgeben
	 * des Ergebniswertes der aktuellen
	 * For-Schleife.
	 * <br/>
	 * Dies entspricht dem Teil 
	 * in den geschwungenen for-Klammern.
	 * 
	 * @param currentLoopValueLevel1 Lauf-Variable der ersten Ebene der For-Schleifen-Hierarchie
	 * @param currentLoopValueLevel2 Lauf-Variable der zweiten Ebene der For-Schleifen-Hierarchie
	 * @return Ergebnis-Wert des aktuellen Durchlaufes der For-Schleife
	 */
	abstract protected R yieldReturn(
			final I currentLoopValueLevel1 ,
			final J currentLoopValueLevel2 );
}
