package util.yield_for.nested;



/**
 * Abstrakte Oberklasse fr
 * die zweite Stufe einer
 * For-Schleife mit yield-return.
 * <br/>
 * Wie bei einem algebraischen Datentyp
 * kann die Hierarchie der For-Schleifen
 * enden ({@link FinalYieldFor3} oder
 * fortgesetzt werden 
 * {@link NestedYieldFor3}.
 * 
 * @param <I> Type-Parameter, Typ der Laufvariable der ersten Stufe, traditionsgemss I genannt
 * @param <J> Type-Parameter, Typ der Laufvariable der zweiten Stufe, traditionsgemss J genannt
 * @param <K> Type-Parameter, Typ der Laufvariable der dritten Stufe, traditionsgemss K genannt
 * @param <R> Type-Parameter, Typ der Return-Wertes des yield-return
 * @author Heiner K&uuml;cker
 */
abstract public class NestedOrFinalYieldFor3<I, J, K, R>
{
	/**
	 * aktuelle Laufvariable
	 */
	/*default*/ K currentLoopValueLevel3;

	/**
	 * Konstruktor.
	 */
	protected NestedOrFinalYieldFor3()
	{
		super();

		init();
	}

	/**
	 * Methode zum Vorbereiten 
	 * eines neuen Durchlaufs,
	 * dies bricht mit dem
	 * Verhalten des normalen
	 * {@link java.util.Iterator}s.
	 */
	protected final void init()
	{
		this.currentLoopValueLevel3 =
				initialValue();
	}

	/**
	 * Zu implementierende Methode, 
	 * welche den initialen Wert der 
	 * Laufvariable Zurckgibt.
	 * <br/>
	 * Dies entspricht dem linken
	 * Teil in den runden for-Klammern.
	 * 
	 * @return initialer Wert der Laufvariable
	 */
	abstract protected K initialValue();

	/**
	 * Methode vorgesehen zum Aufruf aus
	 * {@link NestedYieldFor2#yieldReturn}.
	 * 
	 * @param currentLoopValueLevel1 Lauf-Variable der ersten Ebene der For-Schleifen-Hierarchie
	 * @param currentLoopValueLevel2 Lauf-Variable der zweiten Ebene der For-Schleifen-Hierarchie
	 * @return Ergebnis-Wert des aktuellen Durchlaufes der For-Schleife
	 */
	protected final R outerYieldReturn(
			final I currentLoopValueLevel1 ,
			final J currentLoopValueLevel2 )
	{
		return yieldReturn(
				currentLoopValueLevel1 ,
				currentLoopValueLevel2 ,
				this.currentLoopValueLevel3 );
	}

	/**
	 * Methode vorgesehen zum Aufruf aus
	 * {@link NestedYieldFor2}.
	 * 
	 * @return
	 */
	abstract protected boolean outerIs();

	/**
	 * Zu implementierende Methode
	 * mit dem Code zum Verndern
	 * der Bedingung fr den nchsten
	 * Durchlauf oder Abbruch der
	 * For-Schleife.
	 * <br/>
	 * Dies entspricht dem mittleren
	 * Teil in den runden for-Klammern.
	 * 
	 * @param currentLoopValueLevel3 aktuelles Element, Laufvariable vor der Operation
	 * @return ob die For-Schleife fortgesetzt werden soll
	 */
	abstract protected boolean is(
			final K currentLoopValueLevel3 );

	/**
	 * Methode vorgesehen zum Aufruf aus
	 * {@link NestedYieldFor2}.
	 */
	abstract protected void outerSkip();

	/**
	 * Zu implementierende Methode
	 * mit dem Code der Operation
	 * zum Verndern der Laufvariable
	 * und fr den nchsten
	 * Durchlauf.
	 * <br/>
	 * Dies entspricht dem mittleren
	 * Teil in den runden for-Klammern.
	 * 
	 * @param currentLoopValueLevel3 aktuelles Element, Laufvariable vor der Operation
	 * @return aktuelles Element, Laufvariable nach der Operation
	 */
	abstract protected K skip(
			final K currentLoopValueLevel3 );

	/**
	 * Zu implementierende Methode
	 * mit dem Code zum Zurckgeben
	 * des Ergebniswertes der aktuellen
	 * For-Schleife.
	 * <br/>
	 * Dies entspricht dem Teil 
	 * in den geschwungenen for-Klammern.
	 * 
	 * @param currentLoopValueLevel1 Lauf-Variable der ersten Ebene der For-Schleifen-Hierarchie
	 * @param currentLoopValueLevel2 Lauf-Variable der zweiten Ebene der For-Schleifen-Hierarchie
	 * @param currentLoopValueLevel3 Lauf-Variable der dritten Ebene der For-Schleifen-Hierarchie
	 * @return Ergebnis-Wert des aktuellen Durchlaufes der For-Schleife
	 */
	abstract protected R yieldReturn(
			final I currentLoopValueLevel1 ,
			final J currentLoopValueLevel2 ,
			final K currentLoopValueLevel3 );
}
