package util.yield_for.nested;

import java.util.NoSuchElementException;

/**
 * Implementierungsklasse fr eine
 * mehrstufige For-Schleife mit 
 * yield-return.
 * <br/>
 * Die Hierarchie der For-Schleifen
 * wird mit dieser Klasse fortgesetzt.
 * 
 * @param <I> Type-Parameter, Typ der Laufvariable der ersten Stufe, traditionsgemss I genannt
 * @param <R> Type-Parameter, Typ der Return-Wertes des yield-return
 * @author Heiner K&uuml;cker
 */
public abstract class NestedYieldFor1<I, R>
extends NestedOrFinalYieldFor1<I, R>
{
	/**
	 * Konstruktor.
	 */
	protected NestedYieldFor1()
	{
		super();

		this.subFor =
				subFor();
		
		if ( this.subFor == null )
		{
			throw new IllegalArgumentException(
					"subFor is null" );
		}
	}

	/**
	 * Untergeordnete For-Schleife mit yield-return.
	 */
	private NestedOrFinalYieldFor2<I, ?, R> subFor;

	/**
	 * Zu implementierende Methode zum
	 * Zurckgeben der untergeordneten 
	 * For-Schleife mit yield-return.
	 * 
	 * @return untergeordnete For-Schleife mit yield-return
	 */
	abstract protected NestedOrFinalYieldFor2<I, ?, R> subFor();
	
	/**
	 * @return
	 */
	@Override
	protected final R yieldReturn(
			final I currentLoopValueLevel1 )
	{
		return this.subFor.outerYieldReturn(
				currentLoopValueLevel1 );
	}
	
	/**
	 * @see java.util.Iterator#hasNext()
	 */
	@Override
	public final boolean hasNext()
	{
		return
				this.subFor.outerIs() &&
				is( this.currentLoopValueLevel1 );
	}

	/**
	 * @see java.util.Iterator#next()
	 */
	@Override
	public final R next()
	{
		if ( ! hasNext() )
		{
			throw new NoSuchElementException();
		}

		final R yieldReturnValue =
				this.yieldReturn(
						this.currentLoopValueLevel1 );

		this.subFor.outerSkip();

		if ( ! this.subFor.outerIs() )
		{
			// untergeordnete Schleife neu Initialisieren
			this.subFor.init();

			// aktuelle Schleife Weiterschalten
			this.currentLoopValueLevel1 =
					skip(
							this.currentLoopValueLevel1 );
		}
		
		return yieldReturnValue;
	}

	/**
	 * @see java.util.Iterator#remove()
	 */
	@Override
	public final void remove()
	{
		throw new UnsupportedOperationException();
	}

}
