package util.yield_for.nested;


/**
 * Implementierungsklasse fr eine
 * mehrstufige For-Schleife mit 
 * yield-return.
 * <br/>
 * Die Hierarchie der For-Schleifen
 * wird mit dieser Klasse fortgesetzt.
 * 
 * @param <I> Type-Parameter, Typ der Laufvariable der ersten Stufe, traditionsgemss I genannt
 * @param <J> Type-Parameter, Typ der Laufvariable der zweiten Stufe, traditionsgemss J genannt
 * @param <K> Type-Parameter, Typ der Laufvariable der dritten Stufe, traditionsgemss K genannt
 * @param <R> Type-Parameter, Typ der Return-Wertes des yield-return
 * @author Heiner K&uuml;cker
 */
public abstract class NestedYieldFor3<I, J, K, R>
extends NestedOrFinalYieldFor3<I, J, K, R>
{
	/**
	 * Konstruktor.
	 */
	protected NestedYieldFor3()
	{
		super();

		init();

		this.subFor =
				subFor();
	}

	/**
	 * Untergeordnete For-Schleife mit yield-return.
	 */
	private NestedOrFinalYieldFor4<I, J, K, ?, R> subFor;
	
	/**
	 * Zu implementierende Methode zum
	 * Zurckgeben der untergeordneten 
	 * For-Schleife mit yield-return.
	 * 
	 * @return untergeordnete For-Schleife mit yield-return
	 */
	abstract protected NestedOrFinalYieldFor4<I, J, K, ?, R> subFor();
	
	/**
	 * @return
	 */
	@Override
	protected final R yieldReturn(
			final I currentLoopValueLevel1 ,
			final J currentLoopValueLevel2 ,
			final K currentLoopValueLevel3 )
	{
		return this.subFor.outerYieldReturn(
				currentLoopValueLevel1 ,
				currentLoopValueLevel2 ,
				currentLoopValueLevel3 );
	}
	
	/**
	 * @see NestedOrFinalYieldFor2#outerIs
	 */
	@Override
	final public boolean outerIs()
	{
		return
				this.subFor.outerIs() &&
				this.is( this.currentLoopValueLevel3 );
	}
	
	/**
	 * @see NestedOrFinalYieldFor2#outerSkip
	 */
	@Override
	final protected void outerSkip()
	{
		this.subFor.outerSkip();
		
		if ( ! this.subFor.outerIs() )
		{
			// untergeordnete Schleife neu Initialisieren
			this.subFor.init();
			
			// aktuelle Schleife Weiterschalten
			this.currentLoopValueLevel3 =
					this.skip(
							this.currentLoopValueLevel3 );
		}
	}
}
