package util.yield_for.nested;

import java.util.ArrayList;
import java.util.Arrays;

import junit.framework.Assert;

import org.junit.Test;

import util.yield_for.nested.FinalYieldFor2;
import util.yield_for.nested.NestedOrFinalYieldFor2;
import util.yield_for.nested.NestedYieldFor1;

/**
 * JUnit4-Testcase fr die Klasse
 * {@link FinalYieldFor2}.
 * 
 * @author Heiner K&uuml;cker
 */
public class FinalYieldFor2Test
{
	/**
	 * Test method for {@link FinalYieldFor2}.
	 */
	@Test
	@SuppressWarnings("boxing")
	public final void testDigitsDigits()
	{
		/*
		 * Nachbildung
		 * for ( int i = 0 ; i < 3 ; i++ )
		 *     for ( int j = 0 ; j < 3 ; j++ )
		 *         yield return ( i + "" + j );
		 */
		final NestedYieldFor1<Integer, String> yieldFor =
				new NestedYieldFor1<Integer, String>()
				{
					@Override
					protected Integer initialValue()
					{
						return 0;
					}

					@Override
					protected boolean is(
							final Integer currentLoopValueLevel1 )
					{
						return currentLoopValueLevel1 < 3;
					}

					@Override
					protected Integer skip(
							final Integer currentLoopValueLevel1 )
					{
						return currentLoopValueLevel1 + 1;
					}

					@Override
					protected NestedOrFinalYieldFor2<Integer, ?, String> subFor()
					{
						return
								new FinalYieldFor2<Integer, Integer, String>()
								{
									@Override
									protected Integer initialValue()
									{
										return 0;
									}

									@Override
									protected boolean is(
											final Integer currentLoopValueLevel2 )
									{
										return currentLoopValueLevel2 < 3;
									}

									@Override
									protected Integer skip(
											final Integer currentLoopValueLevel2 )
									{
										return currentLoopValueLevel2 + 1;
									}

									@Override
									protected String yieldReturn(
											final Integer currentLoopValueLevel1 ,
											final Integer currentLoopValueLevel2 )
									{
										return
												currentLoopValueLevel1 + 
												"" + 
												currentLoopValueLevel2;
									}
								};
					}
				};

		final ArrayList<String> actual = new ArrayList<String>();

		while ( yieldFor.hasNext() )
		{
			final String value = yieldFor.next();

			actual.add( value );

			System.out.println( value );
		}

		Assert.assertEquals(
				//expected
				new ArrayList<String>(
						Arrays.asList(
								"00" ,
								"01" ,
								"02" ,
								"10" ,
								"11" ,
								"12" ,
								"20" ,
								"21" ,
								"22" ) ) ,
				actual );
	}

	/**
	 * Test method for {@link FinalYieldFor2}.
	 */
	@Test
	@SuppressWarnings("boxing")
	public final void testLetterDigits()
	{
		final NestedYieldFor1<String, String> yieldFor =
				new NestedYieldFor1<String, String>()
				{
					@Override
					protected String initialValue()
					{
						return "A";
					}

					@Override
					protected boolean is(
							final String currentLoopValueLevel1 )
					{
						return currentLoopValueLevel1.compareTo( "D" ) < 0;
					}

					@Override
					protected String skip(
							final String currentLoopValueLevel1 )
					{
						return String.valueOf(
								(char) ( currentLoopValueLevel1.charAt( 0 ) + 1 ) );
					}

					@Override
					protected NestedOrFinalYieldFor2<String, ?, String> subFor()
					{
						return
								new FinalYieldFor2<String, Integer, String>()
								{
									@Override
									protected Integer initialValue()
									{
										return 0;
									}

									@Override
									protected boolean is(
											final Integer currentLoopValueLevel2 )
									{
										return currentLoopValueLevel2 < 3;
									}

									@Override
									protected Integer skip(
											final Integer currentLoopValueLevel2 )
									{
										return currentLoopValueLevel2 + 1;
									}

									@Override
									protected String yieldReturn(
											final String  currentLoopValueLevel1 ,
											final Integer currentLoopValueLevel2 )
									{
										return
												currentLoopValueLevel1 + 
												"" + 
												currentLoopValueLevel2;
									}
								};
					}
				};

		final ArrayList<String> actual = new ArrayList<String>();

		while ( yieldFor.hasNext() )
		{
			final String value = yieldFor.next();

			actual.add( value );

			System.out.println( value );
		}

		Assert.assertEquals(
				//expected
				new ArrayList<String>(
						Arrays.asList(
								"A0" ,
								"A1" ,
								"A2" ,
								"B0" ,
								"B1" ,
								"B2" ,
								"C0" ,
								"C1" ,
								"C2" ) ) ,
				actual );
	}

	/**
	 * Test method for {@link FinalYieldFor2}.
	 */
	@Test
	@SuppressWarnings("boxing")
	public final void testLowestForNoLoops()
	{
		final NestedYieldFor1<String, String> yieldFor =
				new NestedYieldFor1<String, String>()
				{
					@Override
					protected String initialValue()
					{
						return "A";
					}

					@Override
					protected boolean is(
							final String currentLoopValueLevel1 )
					{
						return currentLoopValueLevel1.compareTo( "D" ) < 0;
					}

					@Override
					protected String skip(
							final String currentLoopValueLevel1 )
					{
						//return String.valueOf( (char) ( currentLoopValueLevel1.charAt( 0 ) + 1 ) );
						// sollte nie aufgerufen werden
						throw new UnsupportedOperationException();
					}

					@Override
					protected NestedOrFinalYieldFor2<String, ?, String> subFor()
					{
						return
								new FinalYieldFor2<String, Integer, String>()
								{
									@Override
									protected Integer initialValue()
									{
										return 0;
									}

									@Override
									protected boolean is(
											final Integer currentLoopValueLevel2 )
									{
										return 
												// kein Durchlauf
												false;
									}

									@Override
									protected Integer skip(
											final Integer currentLoopValueLevel2 )
									{
										//return currentLoopValueLevel2 + 1
										// sollte nie aufgerufen werden
										throw new UnsupportedOperationException();
									}

									@Override
									protected String yieldReturn(
											final String  currentLoopValueLevel1 ,
											final Integer currentLoopValueLevel2 )
									{
										//return currentLoopValueLevel1 + "" + currentLoopValueLevel2;
										// sollte nie aufgerufen werden
										throw new UnsupportedOperationException();
									}
								};
					}
				};

		final ArrayList<String> actual = new ArrayList<String>();

		while ( yieldFor.hasNext() )
		{
			final String value = yieldFor.next();

			actual.add( value );

			System.out.println( value );
		}

		Assert.assertEquals(
				//expected
				new ArrayList<String>() ,
				actual );
	}

}
